

#include <fstream.h>
#include <unistd.h>
#include <string.h>

#include "read_rc.h"
#include "process.h"

#define MAXLENGTH 1024

// Menu parsing parameters
struct functions
{
  char *keyword;
  int type;
} func_config[] =
{
  {"Beep",            F_INACTIVE},
  {"ButtonStyle",     F_INACTIVE},
  {"ClickTime",       F_INACTIVE},
  {"Close",           F_INACTIVE},
  {"ColormapFocus",   F_INACTIVE},
  {"CursorMove",      F_INACTIVE},
  {"Delete",          F_INACTIVE},
  {"DesktopSize",     F_INACTIVE},
  {"Desk",            F_INACTIVE},
  {"Destroy",         F_INACTIVE},
  {"DestroyMenu",     F_INACTIVE},
  {"EdgeScroll",      F_INACTIVE},
  {"EdgeResistance",  F_INACTIVE},
  {"Exec",            F_EXEC    },
  {"Focus",           F_INACTIVE},
  {"Function",        F_FUNCTION},
  {"GotoPage",        F_INACTIVE},
  {"DefaultColors",   F_INACTIVE},
  {"HilightColors",   F_INACTIVE},
  {"StickyColors",    F_INACTIVE},
  {"Iconify",         F_INACTIVE},
  {"IconFont",        F_INACTIVE},
  {"IconPath",        F_INACTIVE},
  {"Key",             F_INACTIVE},
  {"Lower",           F_INACTIVE},
  {"MailFunction",    F_INACTIVE},
  {"Maximize",        F_INACTIVE},
  {"MenuStyle",       F_INACTIVE},
  {"MenuFont",        F_INACTIVE},
  {"Menu",            F_INACTIVE},
  {"ModulePath",      F_INACTIVE},
  {"Module",          F_INACTIVE},
  {"Mouse",           F_INACTIVE},
  {"Move",            F_INACTIVE},
  {"MoveorRaise",     F_INACTIVE},
  {"Nop",             F_NOP,    },
  {"Next",            F_INACTIVE},
  {"None",            F_INACTIVE},
  {"OpaqueMoveSize",  F_INACTIVE},
  {"PixmapPath",      F_INACTIVE},
  {"Popup",           F_POPUP   },
  {"Prev",            F_INACTIVE},
  {"PrintFunction",    F_INACTIVE},
  {"Quit",            F_INACTIVE},
  {"QuitScreen",      F_INACTIVE},
  {"Raise_it",        F_INACTIVE},
  {"RaiseLower",      F_INACTIVE},
  {"Raise",           F_INACTIVE},
  {"Read",            F_INACTIVE},
  {"Recapture",       F_INACTIVE},
  {"Refresh",         F_INACTIVE},
  {"Resize",          F_INACTIVE},
  {"ResizeorRaise",   F_INACTIVE},
  {"Restart",         F_RESTART },
  {"KillModule",      F_INACTIVE},
  {"Scroll",          F_INACTIVE},
  {"Stick",           F_INACTIVE},
  {"Send_WindowList", F_INACTIVE},
  {"Send_ConfigInfo", F_INACTIVE},
  {"set_mask",        F_INACTIVE},
  {"Style",           F_INACTIVE},
  {"Title",           F_INACTIVE},
  {"Wait",            F_INACTIVE},
  {"WindowFont",      F_INACTIVE},
  {"WindowList",      F_INACTIVE},
  {"WindowsDesk",     F_INACTIVE},
  {"XORValue",        F_INACTIVE},
  {"",                F_INACTIVE}
};


int linen=0;

void action();

void eexit()
{
  cout  << " Error in config file line " << linen << "\n" ;
  exit(-1);
}

int strcmplen(char *string1, char *string2)
{
  long len1 = strlen(string1);
  long len2 = strlen(string2);

  long len;
  len = len2-1;
    
  while (len>=0)
  {
    if (string1[len]!=string2[len] )
      return 0;
    len--;
  }
  return 1;
}

main_menu_struct *create_menustruct(char *configfile, int button_num)
{
  ifstream fp;
  char line[MAXLENGTH];
  
  // root is the mouse menu, 
  // anchor the first menu item
  main_menu_struct *root=NULL, *anchor, *menu;
  menu=anchor=new main_menu_struct("");
  
  sub_menu_struct *child;
  int addto=0;
  
  char AddToMenu[] = "AddToMenu";
  char AddTo[]     = "AddTo";
  char Plus[] =      "+";
  char Popup[] =     "Popup";
  char Nop[] =       "Nop";
  char Exec[] =      "Exec";
  
  char Restart[] =   "Restart";
  char Module[] =    "Module";
  char Function[] =  "Function";

  // Not to be used Functions
  char Move[] =        "Move";
  char Resize[] =      "Resize";
  char Iconify[] =     "Iconify";
  char Maximize[] =    "Maximize";
  char Stick[] =       "Stick";
  char Refresh[] =     "Refresh";
  char Raise[] =       "Raise";
  char Lower[] =       "Lower";
  char Close[] =       "Close";
  char Next[] =        "Next";
  char Prev[] =        "Prev";
  char WindowList[] =  "WindowList";
  char Delete[] =      "Delete";
  char Destroy[] =     "Destroy";
  char MailFunction[] = "MailFunction";
  char MoveorRaise[] = "Move-or-Raise";
  char ResizeorRaise[] = "Resize-or-Raise";
  char PrintFunction[] = "PrintFunction";
  
  char Mouse[] =     "Mouse";
  char Root[] =      "R";
  char Menu[] =      "Menu";

  char Mouse_b[2];
  switch(button_num)
  {
    case 1:  Mouse_b[0]= '1'; break;
    case 2:  Mouse_b[0]= '2'; break;
    case 4:  Mouse_b[0]= '4'; break;
    case 5:  Mouse_b[0]= '5'; break;
    
    default: Mouse_b[0]= '3'; break;
  }
  Mouse_b[1]='\0';
  
  fp.open(configfile);

  int i, j;
  while (1)
  {
    char c=' ';
    while(fp.get(c) && (c==' ' || c=='	' || c=='\n'))
      if (c=='\n')
        linen++;

    // end of file
    if (c==' ')
    {
      fp.close();
  
      // finally sort menu items
      return build_tree(root, anchor);
    }
    
    // Read in line
    i=1;
    line[0]=c;
    while(fp.get(line[i]) && line[i]!='\n')
      i++;
 /* cout << "line: " << linen << "\n"; */
   
    if (line[i]=='\n')
    {
      linen++;
      line[i]='\0';
      i=0;
      while (line[i]==' ' || line[i]=='	')  i++;

      //     Mouse  3  R  N  Menu  allpopup  Nop
      //old: Mouse  1  R  A  PopUp "Utilities"

      // Find right mouse root menu
      if(strcmplen(line+i, Mouse))
      {
        i+=strlen(Mouse);
        while (line[i]==' ' || line[i]=='	')  i++;
        if(strcmplen(line+i, Mouse_b))
        {
          i+=strlen(Mouse_b);
          while (line[i]==' ' || line[i]=='	')  i++;
          if(strcmplen(line+i, Root))
          {
            i+=strlen(Root);
            while (line[i]==' ' || line[i]=='	')  i++;

            if (line[i]=='\0')
              eexit();
            // Skip icon setting
            i++;
            
            while (line[i]==' ' || line[i]=='	')  i++;
            if(strcmplen(line+i, Menu) || strcmplen(line+i, Popup))
            {
              char *text;
              if(strcmplen(line+i, Menu))
                i+=strlen(Menu);
              else
                // old format:
                i+=strlen(Popup);
              while (line[i]==' ' || line[i]=='	')  i++;
              j=i;
              while (line[j]!=' ' && line[j]!='	' && line[j]!='\0')  j++;
              text=new char[j-i+1];
              strncpy(text, line+i, j-i);
              text[j-i]='\0';
              root=new main_menu_struct(text);
            /* cout << " ...... " << text << " ..... \n" ; */
              delete text;
            }
          }
        }
      }
      
      // Find current menu
      if(strcmplen(line+i, AddToMenu) || strcmplen(line+i, Popup))
      {
        char *name, *text;
        if(strcmplen(line+i, AddToMenu))
          i+=strlen(AddToMenu);
        else
          // old format:
          i+=strlen(Popup);
        while (line[i]==' ' || line[i]=='	')  i++;
        j=i;
        while (line[j]!=' ' && line[j]!='	' && line[j]!='\0')  j++;
        
         // Set current menu name
        name=new char[j-i+1];
        strncpy(name, line+i, j-i);
        name[j-i]='\0';
        menu->next = new main_menu_struct(name);
        menu=menu->next;
        /* cout << menu->name << " (name)" << "\n"; */
       
        // create child
        menu->child = (int *)new sub_menu_struct();
        child=(sub_menu_struct *)menu->child;
        addto=1;

        // Set current menu title
        if (strcmplen(line+i, menu->name))
        {
          i=j;
          while (line[i]!='"'  && line[i]!='\0')  i++;
          if (line[i]=='\0')
            menu->settext(menu->name);
          else
          {
             // eexit();
            j=++i;
            while (line[j]!='"' && line[j]!='\0') j++;
            text=new char[j-i+1];
            strncpy(text, line+i, j-i);
            text[j-i]='\0';
           
            menu->settext(text);
           /* cout << text << ";" << menu->name << "\n"; */
            delete text;
          }
        }
        delete name;        
      }
      else
      
      // Just skip non-menus
      if(strcmplen(line+i, AddTo))
      {
        addto=0;
      }
      else
      
      // Add to menu structure
      if(strcmplen(line+i, Plus) && addto==1)
      {
          /* cout << " text, icon, type, fkt. : \n"; */

        char *text, *iconname, *function;
        while (line[i]!='"' && line[i]!='\0')  i++;
        j=++i;
        if (line[i]=='\0')
          eexit();
        while (line[j]!='%' && line[j]!='*' && line[j]!='\0' && line[j]!='"') j++;
        text=new char[j-i+1];
        strncpy(text, line+i, j-i);
        text[j-i]='\0';
        
          /* cout << "(" << text << "-\n"; */
        
        if (line[j]=='%')  j++;
        i=j;
        while (line[j]!='%' && line[j]!='"' && line[j]!='*' && line[j]!='\0') j++;
        iconname=new char[j-i+1];
        strncpy(iconname, line+i, j-i);
        iconname[j-i]='\0';
          /* cout << iconname << "-\n"; */
        
        i=j;
        while (line[i]!='"' && line[j]!='\0')  i++;
        if (line[i]=='\0')
          eexit();
        i++;
        while (line[i]==' ' || line[i]=='	')  i++;

        // Different kinds of Menu items
        int type=-1;
        
        int item;
        for (item=0; (func_config[item].keyword[0]!='\0' &&
                       !strcmplen(line+i, func_config[item].keyword)); item++)  {}
        if (strcmp(func_config[item].keyword, ""))
        {
    /* cout << func_config[item].keyword << " keywd\n\n"; */
          type=func_config[item].type;
          switch (type)
          {
          
            case F_NOP:
              function=new char[1];
              function[0]='\0';
              break;
  
            case F_INACTIVE:
              function=new char[1];
              function[0]='\0';
              break;
  
            case F_POPUP:
              i+=strlen(Popup);
              while (line[i]==' ' || line[i]=='	')  i++;
              j=i;
              while (line[j]!=' ' && line[j]!='	' && line[j]!='\0')  j++;
              function=new char[j-i+1];
              strncpy(function, line+i, j-i);
              function[j-i]='\0';
             break;
  
            case F_EXEC:
              i+=strlen(Exec);
              while (line[i]==' ' || line[i]=='	')  i++;
              j=i;
              while (line[j]!='\0')  j++;
              function=new char[j-i+1];
              strncpy(function, line+i, j-i);
              function[j-i]='\0';
              // &action=&exec_function;
              break;
  
            case F_FUNCTION:
              i+=strlen(Function);
              while (line[i]==' ' || line[i]=='	')  i++;
              j=i;
              while (line[j]!='\0')  j++;
              function=new char[j-i+1];
              strncpy(function, line+i, j-i);
              function[j-i]='\0';
              // &action=&ComplexFunction;
              break;
  
            case F_RESTART:
              i+=strlen(Restart);
              while (line[i]==' ' || line[i]=='	')  i++;
              j=i;
              while (line[j]!='\0')  j++;
              function=new char[j-i+1];
              strncpy(function, line+i, j-i);
              function[j-i]='\0';
              // &action=&restart_function;
              break;
          }
        }
        
        // New child menu
        child->next = new sub_menu_struct(text, type, 
            iconname, function);
        if (strlen(iconname)>0) child->seticon(iconname);
        child=child->next;

        delete text;
        delete iconname;
        delete function;
      }
    }
    else
    // old format:
    if(strcmplen(line+i, Plus) && addto==1)
    {
      while (line[i]==' ' || line[i]=='	')  i++;

      // Different kinds of Menu items
      int type=-1;
      
      int item;
      for (item=0; (func_config[item].keyword[0]!='\0' &&
                     !strcmplen(line+i, func_config[item].keyword)); item++)  {}
      if (strcmp(func_config[item].keyword, ""))
      {
  /* cout << func_config[item].keyword << " keywd\n\n"; */
        type=func_config[item].type;
        switch (type)
        {
        
          case F_NOP:
            break;
        }
      }
    }
    else
      break;
 }

  fp.close();
  
   /* cout << "\n\n"; */
  
  // finally sort menu items
  return build_tree(root, anchor);
}


main_menu_struct *build_tree(main_menu_struct *root, main_menu_struct *anchor)
{
  main_menu_struct *menu=anchor;

  if (root==NULL)
  {
    cout << "Couldn't create any menu.\n";
    exit(1);
  }
  else
  if (root->name==NULL)
   if (menu->name==NULL)
  {
    cout << "Couldn't create any menu.\n";
    exit(1);
  }

  while(strcmp(root->name, menu->name) && menu->next!=NULL)
    menu=menu->next;
  root->child=(int *)menu;
  
 /* cout << "\n\n";
 cout << " menu, name, text,  : (" << menu->name << "-" << menu->text << ")\n" ;
 cout << " root, name, text,  : (" << root->name << ")\n" ; */

  menu=anchor;
  // Eliminate item after item
  while((menu=anchor->next) != NULL)
  {
    anchor->next=menu->next;
    
    main_menu_struct *tmpmenu=menu;
    
    // Run trough list
    while((tmpmenu=tmpmenu->next) != NULL)
    {
      // Append element below or on top of another

      sub_menu_struct *child;
      
      child=(sub_menu_struct *)tmpmenu->child;
      while((child=child->next) != NULL)
        if (child->type==F_POPUP && 
            !strcmp(child->function, menu->name))
      {
        child->child=menu;
         /* cout << child->function << "; " << menu->name << ";\n"; */
      }
        
      // try vice versa
      child=(sub_menu_struct *)menu->child;
      while((child=child->next) != NULL)
      if (child->type==F_POPUP && 
          !strcmp(child->function, tmpmenu->name))
      {
        child->child=tmpmenu;
        /* cout << child->function << "!" << tmpmenu->name << "!\n"; */
      }
      
      // Root menu
      if (!strcmp(root->name, menu->name))
        root->child=(int *)menu;
    
    }
  }


  // Maybe create misc. menu
  sub_menu_struct *misc=new sub_menu_struct();
  sub_menu_struct *misc_child=new sub_menu_struct();
  
  misc->child=new main_menu_struct("misc");
  (sub_menu_struct *)misc->child->child=misc_child;
  sub_menu_struct *menux=(sub_menu_struct *)((main_menu_struct *)root->child)->child;
  int do_misc=0;
  if (menux!=NULL)
  while (menux->next!=NULL)
    if (menux->next->type!=F_POPUP)
    {
      do_misc=1;
      misc_child->next=menux->next;
      menux->next=menux->next->next;
      misc_child=misc_child->next;
    }
    else
    menux=menux->next;
  if (do_misc) 
  {
    misc_child->next=NULL;
    misc->text=strdup("Main"); 
    misc->function=strdup("misc");
    misc->type=F_POPUP;
    misc->next=((sub_menu_struct *)((main_menu_struct *)root->child)->child)->next;
    ((sub_menu_struct *)((main_menu_struct *)root->child)->child)->next=misc;
  }
  
  
   /* cout << "\n\n"; *
   menux=(sub_menu_struct *)((main_menu_struct *)root->child)->child;
  while((menux=menux->next) != NULL)
  {
/* cout << menux->function << "!" << menux->text << "!\n"; *
    
    if (menux->type==F_POPUP)
    {
      sub_menu_struct *child=(sub_menu_struct *)menux->child->child;
      while((child=child->next) != NULL)
      {
/* cout << "--" << child->type << child->function << "!" << child->text << "!\n"; *
      
        if (child->type==F_POPUP)
        {
          sub_menu_struct *childchild=(sub_menu_struct *)child->child->child;
          /* while((childchild=childchild->next) != NULL)
             cout << "----" << childchild->function << "!" << childchild->text << "!\n"; *
        }
      }
    }
  }*/
  
  return root;

}
